// JSON polyfill for ExtendScript
if (typeof JSON === 'undefined') {
    JSON = {};
}

// stringify polyfill (keep existing)
if (typeof JSON.stringify !== 'function') {
    JSON.stringify = function(obj) {
        var t = typeof obj;
        if (t === "string") return '"' + obj.replace(/\\/g, '\\\\').replace(/"/g, '\\"').replace(/\n/g, '\\n').replace(/\r/g, '\\r') + '"';
        if (t === "number" || t === "boolean") return "" + obj;
        if (t === "object") {
            if (obj === null) return "null";
            if (obj instanceof Array) {
                var str = '[';
                for (var i = 0; i < obj.length; i++) {
                    if (i > 0) str += ',';
                    str += JSON.stringify(obj[i]);
                }
                return str + ']';
            }
            var str = '{';
            var first = true;
            for (var p in obj) {
                if (obj.hasOwnProperty(p)) {
                    if (!first) str += ',';
                    str += '"' + p + '":' + JSON.stringify(obj[p]);
                    first = false;
                }
            }
            return str + '}';
        }
        return '""';
    };
}

// parse polyfill (corrected regex escaping)
if (typeof JSON.parse !== 'function') {
    JSON.parse = function(text, reviver) {
        var j;

        function walk(holder, key) {
            var k, v, value = holder[key];
            if (value && typeof value === 'object') {
                for (k in value) {
                    if (Object.prototype.hasOwnProperty.call(value, k)) {
                        v = walk(value, k);
                        if (v !== undefined) {
                            value[k] = v;
                        } else {
                            delete value[k];
                        }
                    }
                }
            }
            return reviver.call(holder, key, value);
        }

        text = String(text);
        // Regular expression check for validity (safer parsing approach)
        // This regex is complex and ensures basic JSON structure adherence.
        // It's crucial for preventing potential code injection if the input isn't true JSON.
        var cx = /[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g;
        cx.lastIndex = 0;
        if (cx.test(text)) {
            text = text.replace(cx, function (a) {
                return '\\u' + ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
            });
        }

        if (/^[\],:{}\s]*$/.test(text.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g, '@')
            .replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g, ']')
            .replace(/(?:^|:|,)(?:\s*\[)+/g, ''))) {

            try {
                 // Use Function constructor for safer evaluation than direct eval
                j = (new Function('return ' + text))();
            } catch (e) {
                 throw new SyntaxError('JSON.parse: ' + e.message);
            }

            return typeof reviver === 'function'
                ? walk({'': j}, '')
                : j;
        }
        throw new SyntaxError('JSON.parse');
    };
}

// Utility function to pad numbers with leading zeros
function padNumber(num) {
    return (num < 10 ? "0" : "") + num;
}

// ExtendScript code for Premiere Pro interaction
function getSequenceFrameRate() {
    try {
        $.writeln("Starting getSequenceFrameRate");
        
        var activeSequence = app.project.activeSequence;
        if (!activeSequence) {
            $.writeln("No active sequence found");
            return 29.97; // Default fallback
        }

        // Get timebase and convert to frame rate
        var timebase = activeSequence.timebase;
        $.writeln("Got timebase: " + timebase);

        // Convert timebase to fps
        var TICKS_PER_SECOND = 254016000000;
        var fps = TICKS_PER_SECOND / timebase;
        
        // Round to common frame rates
        if (Math.abs(fps - 23.976) < 0.01) fps = 23.976;
        else if (Math.abs(fps - 29.97) < 0.01) fps = 29.97;
        else if (Math.abs(fps - 59.94) < 0.01) fps = 59.94;
        else fps = Math.round(fps * 1000) / 1000; // Round to 3 decimal places
        
        $.writeln("Calculated sequence frame rate: " + fps);
        return fps;
    } catch (e) {
        $.writeln("Error in getSequenceFrameRate: " + e.toString());
        return 29.97; // Default fallback
    }
}

function getCurrentTimecode() {
    try {
        $.writeln("Starting getCurrentTimecode");
        
        var activeSequence = app.project.activeSequence;
        if (!activeSequence) {
            $.writeln("No active sequence found");
            return JSON.stringify(null);
        }

        // Get the current time from the sequence
        var playerPosition = activeSequence.getPlayerPosition();
        $.writeln("Got player position: " + playerPosition.ticks);
        
        // Get sequence frame rate
        var frameRate = getSequenceFrameRate();
        var isDropFrame = frameRate === 29.97 || frameRate === 59.94;
        var timeBase = Math.round(frameRate); // 30 for 29.97, 60 for 59.94
        
        $.writeln("Frame rate: " + frameRate + ", isDropFrame: " + isDropFrame);
        
        // Convert ticks to seconds
        var TICKS_PER_SECOND = 254016000000;
        var seconds = playerPosition.ticks / TICKS_PER_SECOND;
        $.writeln("Seconds: " + seconds);
        
        // Calculate initial frame number
        var frameNumber = Math.round(seconds * frameRate);
        $.writeln("Initial frame number: " + frameNumber);
        
        if (isDropFrame) {
            // Drop frame calculation matching TimecodeUtils.framesToDropFrameTimecode
            var dropFrames = Math.round(frameRate * 0.066666); // Number of frames to drop on the minute marks
            var framesPerHour = Math.round(frameRate * 60 * 60);
            var framesPer24Hours = framesPerHour * 24;
            var framesPer10Minutes = Math.round(frameRate * 60 * 10);
            var framesPerMinute = (Math.round(frameRate) * 60) - dropFrames;
            
            // Handle negative time
            while (frameNumber < 0) {
                frameNumber = framesPer24Hours + frameNumber;
            }
            
            // Handle rollover
            frameNumber = frameNumber % framesPer24Hours;
            
            var d = Math.floor(frameNumber / framesPer10Minutes);
            var m = frameNumber % framesPer10Minutes;
            
            if (m > dropFrames) {
                frameNumber = frameNumber + (dropFrames * 9 * d) + dropFrames * Math.floor((m - dropFrames) / framesPerMinute);
            } else {
                frameNumber = frameNumber + dropFrames * 9 * d;
            }
            
            $.writeln("Adjusted frame number for drop frame: " + frameNumber);
        }
        
        // Calculate timecode components
        var frRound = Math.round(frameRate);
        var frames = frameNumber % frRound;
        var seconds = Math.floor(frameNumber / frRound) % 60;
        var minutes = Math.floor(Math.floor(frameNumber / frRound) / 60) % 60;
        var hours = Math.floor(Math.floor(Math.floor(frameNumber / frRound) / 60) / 60);
        
        $.writeln("Timecode components: " + hours + ":" + minutes + ":" + seconds + ":" + frames);
        
        // Format the timecode
        var separator = isDropFrame ? ";" : ":";
        var formattedTimecode = padNumber(hours) + ":" + padNumber(minutes) + ":" + padNumber(seconds) + separator + padNumber(frames);
        
        $.writeln("Formatted timecode: " + formattedTimecode);
        return JSON.stringify({
            timecode: formattedTimecode,
            components: {
                hours: hours,
                minutes: minutes,
                seconds: seconds,
                frames: frames,
                isDropFrame: isDropFrame,
                frameRate: frameRate
            }
        });
    } catch (e) {
        $.writeln("Error in getCurrentTimecode: " + e.toString());
        return JSON.stringify(null);
    }
}

function getMarkersTimecode() {
    try {
        $.writeln("Starting getMarkersTimecode");
        
        var activeSequence = app.project.activeSequence;
        if (!activeSequence) {
            $.writeln("No active sequence found");
            return JSON.stringify({ error: "No active sequence found" });
        }

        // Get sequence details
        var sequenceName = activeSequence.name;
        var frameRate = getSequenceFrameRate();
        var isDropFrame = frameRate === 29.97 || frameRate === 59.94;
        
        $.writeln("Sequence name: " + sequenceName);
        $.writeln("Frame rate: " + frameRate);
        $.writeln("Is drop frame: " + isDropFrame);
        
        // Get markers
        var markers = activeSequence.markers;
        if (!markers || markers.numMarkers === 0) {
            $.writeln("No markers found in sequence");
            return JSON.stringify({ error: "No markers found in sequence" });
        }

        $.writeln("Number of markers: " + markers.numMarkers);

        var markersData = [];
        var TICKS_PER_SECOND = 254016000000;

        for (var i = 0; i < markers.numMarkers; i++) {
            try {
                var marker = markers[i];
                $.writeln("\nProcessing marker " + (i + 1));
                
                // Convert ticks to seconds
                var seconds = marker.start.ticks / TICKS_PER_SECOND;
                $.writeln("Marker seconds: " + seconds);
                
                // Calculate frames without rounding initially
                var totalFrames = seconds * frameRate;
                $.writeln("Raw total frames: " + totalFrames);
                
                // Handle drop frame timecode first
                if (isDropFrame) {
                    var dropFrames = Math.round(frameRate * 0.066666); // Number of frames to drop per minute
                    var totalMinutes = Math.floor(seconds / 60);
                    var droppedFrames = dropFrames * (totalMinutes - Math.floor(totalMinutes / 10));
                    totalFrames = totalFrames + droppedFrames;
                    $.writeln("Adjusted frames for drop frame: " + totalFrames);
                }
                
                // Now round the total frames after drop frame adjustment
                totalFrames = Math.round(totalFrames);
                $.writeln("Rounded total frames: " + totalFrames);
                
                // Calculate timecode components
                var hours = Math.floor(totalFrames / (Math.round(frameRate) * 3600));
                var remainingFrames = totalFrames % (Math.round(frameRate) * 3600);
                
                var minutes = Math.floor(remainingFrames / (Math.round(frameRate) * 60));
                remainingFrames = remainingFrames % (Math.round(frameRate) * 60);
                
                var secs = Math.floor(remainingFrames / Math.round(frameRate));
                var frames = Math.floor(remainingFrames % Math.round(frameRate));
                
                $.writeln("Timecode components: " + hours + ":" + minutes + ":" + secs + ":" + frames);
                
                markersData.push({
                    name: marker.name || "Marker " + (i + 1),
                    timecode: {
                        hours: hours,
                        minutes: minutes,
                        seconds: secs,
                        frames: frames,
                        isDropFrame: isDropFrame,
                        frameRate: frameRate
                    },
                    startTime: seconds,
                    startTicks: marker.start.ticks
                });
                
                $.writeln("Successfully processed marker " + (i + 1));
            } catch (markerError) {
                $.writeln("Error processing marker " + (i + 1) + ": " + markerError.toString());
            }
        }

        var result = {
            success: true,
            markers: markersData,
            sequenceName: sequenceName,
            isDropFrame: isDropFrame
        };
        
        $.writeln("Final result: " + JSON.stringify(result));
        return JSON.stringify(result);
    } catch (error) {
        $.writeln("Error in getMarkersTimecode: " + error.toString());
        return JSON.stringify({ error: error.toString() });
    }
}

// --- Base64 Encoding/Decoding Polyfill for ExtendScript --- 
// (Moved here to be defined before use)

var base64Chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

function base64Encode(str) {
    var out = "", i = 0, len = str.length, c1, c2, c3;
    while (i < len) {
        c1 = str.charCodeAt(i++) & 0xff;
        if (i == len) {
            out += base64Chars.charAt(c1 >> 2);
            out += base64Chars.charAt((c1 & 0x3) << 4);
            out += "==";
            break;
        }
        c2 = str.charCodeAt(i++);
        if (i == len) {
            out += base64Chars.charAt(c1 >> 2);
            out += base64Chars.charAt(((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4));
            out += base64Chars.charAt((c2 & 0xF) << 2);
            out += "=";
            break;
        }
        c3 = str.charCodeAt(i++);
        out += base64Chars.charAt(c1 >> 2);
        out += base64Chars.charAt(((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4));
        out += base64Chars.charAt(((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6));
        out += base64Chars.charAt(c3 & 0x3F);
    }
    return out;
}

function base64Decode(str) {
    var out = "", i = 0, len = str.length, c1, c2, c3, c4;
    var lookup = {};
    for (var j = 0; j < base64Chars.length; j++) {
        lookup[base64Chars[j]] = j;
    }

    while (i < len) {
        c1 = lookup[str.charAt(i++)];
        c2 = lookup[str.charAt(i++)];
        c3 = lookup[str.charAt(i++)];
        c4 = lookup[str.charAt(i++)];

        out += String.fromCharCode((c1 << 2) | (c2 >> 4));
        if (c3 != 64 && c3 !== undefined) { // Check for padding '=' which maps to 64 or undefined
            out += String.fromCharCode(((c2 & 15) << 4) | (c3 >> 2));
        }
        if (c4 != 64 && c4 !== undefined) { // Check for padding '=' which maps to 64 or undefined
            out += String.fromCharCode(((c3 & 3) << 6) | c4);
        }
    }
    // Simple UTF-8 decode (might need improvement for full multi-byte chars)
    try {
       return decodeURIComponent(escape(out));
    } catch(e) {
       $.writeln("[Base64 Decode] Warning: Could not decode UTF-8 properly: " + e);
       return out; // Return raw string if UTF decode fails
    }
}

// --- License File Management Functions ---

function getLicenseFilePath() {
    try {
        // Using Folder.userData provides a consistent user-specific location
        var userDataFolder = Folder.userData;
        if (!userDataFolder.exists) {
            userDataFolder.create();
        }
        
        // Create a subfolder for your extension's data
        var appDataFolder = new Folder(userDataFolder.fsName + "/TimeWeaveCalculator");
        if (!appDataFolder.exists) {
            appDataFolder.create();
        }
        
        // Define the license file name
        var licenseFileName = "twc_lic.dat"; // Use a slightly obscure name
        var licenseFilePath = appDataFolder.fsName + "/" + licenseFileName;
        
        $.writeln("[License File] Path: " + licenseFilePath);
        return licenseFilePath;
    } catch (e) {
        $.writeln("[License File] Error getting file path: " + e.toString());
        return null;
    }
}

function writeLicenseData(jsonData) {
    try {
        var filePath = getLicenseFilePath();
        if (!filePath) return JSON.stringify({ success: false, error: "Could not get file path" });
        
        var licenseFile = new File(filePath);
        licenseFile.encoding = "UTF-8"; // Set encoding
        
        if (licenseFile.open("w")) {
            // Basic obfuscation (Base64 encoding)
            var encodedData = base64Encode(jsonData);
            licenseFile.write(encodedData);
            licenseFile.close();
            $.writeln("[License File] Successfully wrote data to: " + filePath);
            return JSON.stringify({ success: true });
        } else {
            $.writeln("[License File] Error opening file for writing: " + licenseFile.error);
            return JSON.stringify({ success: false, error: "Could not open file for writing: " + licenseFile.error });
        }
    } catch (e) {
        $.writeln("[License File] Error writing data: " + e.toString());
        return JSON.stringify({ success: false, error: "Exception writing data: " + e.toString() });
    }
}

function readLicenseData() {
    try {
        var filePath = getLicenseFilePath();
        if (!filePath) return JSON.stringify({ success: false, error: "Could not get file path", data: null });

        var licenseFile = new File(filePath);
        if (!licenseFile.exists) {
            $.writeln("[License File] File does not exist: " + filePath);
            return JSON.stringify({ success: true, data: null }); // Not an error if file doesn't exist
        }
        
        licenseFile.encoding = "UTF-8";
        
        if (licenseFile.open("r")) {
            var encodedData = licenseFile.read();
            licenseFile.close();
            
            if (encodedData) {
                // Decode the data
                var decodedJson = base64Decode(encodedData);
                $.writeln("[License File] Successfully read and decoded data from: " + filePath);
                 // Attempt to parse the JSON
                 try {
                    var parsedData = JSON.parse(decodedJson);
                    return JSON.stringify({ success: true, data: parsedData });
                 } catch (jsonError) {
                    $.writeln("[License File] Error parsing JSON data: " + jsonError.toString());
                    return JSON.stringify({ success: false, error: "Error parsing stored data", data: null });
                 }
            } else {
                $.writeln("[License File] File was empty: " + filePath);
                return JSON.stringify({ success: true, data: null }); // Empty file is like no file
            }
        } else {
            $.writeln("[License File] Error opening file for reading: " + licenseFile.error);
            return JSON.stringify({ success: false, error: "Could not open file for reading: " + licenseFile.error, data: null });
        }
    } catch (e) {
        $.writeln("[License File] Error reading data: " + e.toString());
        return JSON.stringify({ success: false, error: "Exception reading data: " + e.toString(), data: null });
    }
}

// --- Function to get raw system info for JS hashing ---
function getRawSystemInfo() {
    try {
        // Use File.fs (returns "Windows" or "Macintosh") as a base identifier
        var fsType = File.fs;
        // We could potentially add other less volatile info if needed,
        // but just the OS type is a starting point for a consistent ID base.
        var info = fsType + "_UserDataFolder:" + Folder.userData.fsName;
        $.writeln("[System Info] Raw info: " + info);
        return info;
    } catch (e) {
        $.writeln("[System Info] Error getting raw info: " + e.toString());
        return "unknown_fs_error"; // Indicate failure
    }
} 