/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 * Copyright 2024 Adobe
 * All Rights Reserved.
 *
 * NOTICE: Adobe permits you to use, modify, and distribute this file in
 * accordance with the terms of the Adobe license agreement accompanying
 * it. If you have received this file from a source other than Adobe,
 * then your use, modification, or distribution of it requires the prior
 * written permission of Adobe.
 **************************************************************************/

const csInterface = new CSInterface();

// License Management System
class LicenseManager {
    constructor() {
        this.activateApiEndpoint = 'https://timeweave.cc/api/validate-license';
        this.deactivateApiEndpoint = 'https://timeweave.cc/api/deactivate-license';
        this.machineId = null;
        this.isLicensed = false;
        this.currentLicenseKey = null;
        this.offlineGracePeriodMs = 30 * 24 * 60 * 60 * 1000;
        this.isOfflineMode = false;
        this.onStatusChange = null;
        this.currentLicenseType = null;
        this.currentLicenseExpiresAt = null;
        
        this.initializeLicenseUI();
        this.initializeTabs();
        this.showCalculatorTab();
    }
    
    async callExtendScript(functionName, args = []) {
        return new Promise((resolve, reject) => {
            const script = `${functionName}(${args.map(JSON.stringify).join(', ')});`;
            console.log(`[ExtendScript Call] Running: ${script.substring(0, 100)}...`);
            csInterface.evalScript(script, (result) => {
                console.log(`[ExtendScript Result] ${functionName}:`, result);
                // Check for ExtendScript errors or specific non-JSON returns
                if (result === undefined || result === "undefined" || result.indexOf("EvalScript error.") !== -1) {
                     console.error(`[ExtendScript Error] ${functionName} returned error or undefined:`, result);
                     // Reject with a specific error type might be useful
                     reject(new Error(`ExtendScript execution failed for ${functionName}`));
                     return;
                }
                
                // Proceed with JSON parsing for expected results
                try {
                    const parsedResult = JSON.parse(result);
                    resolve(parsedResult);
                } catch (e) {
                    console.error(`[ExtendScript Error] Failed to parse JSON result from ${functionName}:`, result, e);
                    reject(new Error(`Failed to parse ExtendScript result: ${e.message}`));
                }
            });
        });
    }

    async initialize() {
        console.log("[License Init] Starting background initialization...");
        this.isOfflineMode = false;
        await this.generateMachineId();
        await this.checkExistingLicense();
        console.log(`[License Init] Background check complete. Status Determined. Licensed: ${this.isLicensed}`);
    }

    async sha256(message) {
        // Create hash of message using Web Crypto API
        const msgBuffer = new TextEncoder().encode(message);
        const hashBuffer = await crypto.subtle.digest('SHA-256', msgBuffer);
        const hashArray = Array.from(new Uint8Array(hashBuffer));
        return hashArray.map(b => b.toString(16).padStart(2, '0')).join('');
    }

    async generateMachineId() {
        console.log("[License Init] Getting raw system info from ExtendScript...");
        try {
            // Use a direct evalScript call for this simple string return
            const rawInfo = await new Promise((resolve) => {
                 csInterface.evalScript('getRawSystemInfo()', (result) => {
                     // Handle potential undefined/error strings from ExtendScript
                     resolve(result && result !== "unknown_error" ? result : `fallback_${Date.now()}`); 
                 });
            });
            console.log("[License Init] Raw system info received:", rawInfo);
            
            // Hash the raw info on the JS side
            this.machineId = await this.sha256(rawInfo);
            
            console.log("[License Init] Machine ID generated:", this.machineId);
        } catch (error) {
            console.error('Error generating machine ID:', error);
            // Generate a less reliable fallback if ExtendScript call fails entirely
            this.machineId = await this.sha256(`fallback_${navigator.userAgent}_${Date.now()}`);
            console.log("[License Init] Using fallback Machine ID:", this.machineId);
        }
    }

    initializeLicenseUI() {
        this.licenseSection = document.getElementById('license-section');
        this.licenseStatusText = document.getElementById('license-status-text');
        this.licenseInput = document.getElementById('license-key');
        this.activateButton = document.getElementById('activate-license');
        this.deactivateButton = document.getElementById('deactivate-license');
        
        this.licenseTabStatus = document.getElementById('license-tab-status');
        this.licenseTabKey = document.getElementById('license-tab-key');
        this.licenseTabType = document.getElementById('license-tab-type');
        this.licenseTabExpires = document.getElementById('license-tab-expires');
        this.licenseExpiryRow = document.querySelector('.license-expiry-row');
        this.startTrialButton = document.getElementById('start-trial');

        if (this.activateButton) {
            this.activateButton.addEventListener('click', () => {
                this.validateLicense(this.licenseInput.value);
            });
        }

        if (this.deactivateButton) {
            this.deactivateButton.addEventListener('click', () => {
                if (confirm("Are you sure you want to deactivate the license on this device?")) {
                    this.deactivateLicense();
                }
            });
        }

        if (this.licenseInput) {
            this.licenseInput.addEventListener('keypress', (e) => {
                if (e.key === 'Enter') {
                    e.preventDefault();
                    this.validateLicense(this.licenseInput.value);
                }
            });
        }

        if (this.startTrialButton) {
            this.startTrialButton.addEventListener('click', () => {
                csInterface.openURLInDefaultBrowser('https://timeweave.cc/plugin');
            });
        }
    }

    initializeTabs() {
        this.tabNavigation = document.getElementById('tab-navigation');
        this.tabContent = document.getElementById('tab-content');
        this.tabButtons = document.querySelectorAll('.tab-button');
        this.tabPanels = document.querySelectorAll('.tab-panel');

        this.tabButtons.forEach(button => {
            button.addEventListener('click', () => this.switchTab(button.dataset.tab));
        });
    }

    switchTab(tabId) {
        this.tabButtons.forEach(button => {
            button.classList.toggle('active', button.dataset.tab === tabId);
        });
        this.tabPanels.forEach(panel => {
            panel.classList.toggle('active', panel.id === `tab-panel-${tabId}`);
        });
        console.log(`[Tabs] Switched to tab: ${tabId}`);
    }

    showCalculatorTab() {
        if (this.tabNavigation) this.tabNavigation.classList.remove('hidden');
        if (this.tabContent) this.tabContent.classList.remove('hidden');
        this.switchTab('calculator');
        if (this.licenseSection) this.licenseSection.classList.add('hidden');
    }

    showActivationSection() {
        if (this.licenseSection) this.licenseSection.classList.remove('hidden');
        if (this.tabNavigation) this.tabNavigation.classList.add('hidden');
        if (this.tabContent) this.tabContent.classList.add('hidden');
    }
    
    _updateStatus(isValid, message, isOffline = false, licenseType = null, expiresAt = null) {
        console.log(`[_updateStatus] Firing status change. Valid: ${isValid}, Offline: ${isOffline}, Type: ${licenseType}, Expires: ${expiresAt}, Message: ${message}`);
        const previousLicensedState = this.isLicensed;
        this.isLicensed = isValid;
        this.isOfflineMode = isValid && isOffline;
        this.currentLicenseType = isValid ? licenseType : null;
        this.currentLicenseExpiresAt = isValid ? expiresAt : null;
        
        if (!isValid) {
            this.currentLicenseKey = null; 
            this.currentLicenseType = null;
            this.currentLicenseExpiresAt = null;
        }
        
        if (isValid) {
            if (!previousLicensedState) {
                 this.showCalculatorTab();
            }
        } else {
            if (previousLicensedState) {
                this.showActivationSection();
            }
        }

        if (typeof this.onStatusChange === 'function') {
            const status = isValid ? (isOffline ? 'offline' : 'active') : 'inactive';
            this.onStatusChange(status, message);
        } else {
            console.warn("LicenseManager: onStatusChange callback not set!");
        }
        
        this.updateLicenseTabDisplay(isValid ? (isOffline ? 'offline' : 'active') : 'inactive', message, this.currentLicenseType, this.currentLicenseExpiresAt);
    }
    
    updateLicenseTabDisplay(status, message, licenseType, expiresAt) {
        const isValid = (status === 'active' || status === 'offline');
        const isOffline = (status === 'offline');
        
        if (this.licenseTabStatus) {
            let statusText = 'Inactive';
            let statusColor = '#ff7b7b';
            if (isValid) {
                statusText = isOffline ? `Active (Offline - ${message.split('until ')[1] || '...'})` : 'Active';
                statusColor = isOffline ? '#ffcc00' : '#7bff7b';
            }
            this.licenseTabStatus.textContent = statusText;
            this.licenseTabStatus.style.color = statusColor;
        }
        if (this.licenseTabKey) {
            this.licenseTabKey.textContent = isValid ? this.currentLicenseKey : '-';
        }
        
        if (this.licenseTabType) {
            let typeText = '-';
            if (isValid && licenseType) {
                if (licenseType.toLowerCase() === 'trial') {
                    typeText = 'Trial';
                } else {
                    typeText = licenseType.charAt(0).toUpperCase() + licenseType.slice(1);
                }
            }
            this.licenseTabType.textContent = typeText;
        }
        
        if (this.licenseTabExpires && this.licenseExpiryRow) {
            if (isValid && (licenseType === 'subscription' || licenseType === 'trial') && expiresAt) {
                 try {
                     const expiryDate = new Date(expiresAt);
                     const formattedDate = `${expiryDate.getMonth() + 1}/${expiryDate.getDate()}/${expiryDate.getFullYear()}`;
                     this.licenseTabExpires.textContent = formattedDate;
                     this.licenseExpiryRow.classList.remove('hidden');
                 } catch (e) {
                     console.error("Error formatting expiry date:", e);
                     this.licenseTabExpires.textContent = 'Invalid Date';
                     this.licenseExpiryRow.classList.remove('hidden');
                 }
            } else {
                this.licenseTabExpires.textContent = '-';
                this.licenseExpiryRow.classList.add('hidden');
            }
        }
        
        if (this.deactivateButton) {
            this.deactivateButton.disabled = !isValid;
            this.deactivateButton.style.opacity = isValid ? 1 : 0.5;
            this.deactivateButton.style.cursor = isValid ? 'pointer' : 'not-allowed';
        }
        
        if (this.licenseStatusText && !isValid) {
            this.licenseStatusText.textContent = message;
            this.licenseStatusText.parentElement.parentElement.className = `license-status invalid`;
        }
    }

    async validateLicense(licenseKey) {
        if (!this.machineId) {
            await this.generateMachineId();
        }

        if (!licenseKey) {
            this._updateStatus(false, 'Please enter a license key.', false, null, null);
            return false; 
        }
        
        const keyToValidate = licenseKey.trim();

        this.updateLicenseTabDisplay('validating', 'Validating...', null, null); 

        console.log(`[License Validate] Validating key: ${keyToValidate} ...`);

        try {
            const response = await fetch(this.activateApiEndpoint, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                },
                body: JSON.stringify({
                    licenseKey: keyToValidate,
                    machineId: this.machineId
                })
            });

            const data = await response.json();
            
            if (response.ok && data.valid) { 
                this.isOfflineMode = false; 
                this.currentLicenseKey = keyToValidate;
                this.currentLicenseType = data.license_type;
                this.currentLicenseExpiresAt = data.expires_at;
                await this.storeLicense(keyToValidate, data.license_type, data.expires_at); 
                this._updateStatus(true, 'License Active', false, data.license_type, data.expires_at); 
                return true;
            } else {
                this._updateStatus(false, data.message || 'Invalid License', false, null, null); 
                return false; 
            }
        } catch (error) {
             console.error("[License Validate] Network error...", error);
             this._updateStatus(false, 'Validation Failed (Network Error?)', false, null, null); 
             return false; 
        }
    }

    async storeLicense(licenseKey, licenseType, expiresAt) {
        const now = Date.now();
        const validUntil = now + this.offlineGracePeriodMs;
        const dataToStore = {
            key: licenseKey,
            machineId: this.machineId,
            timestamp: now,
            validUntil: validUntil,
            type: licenseType || 'unknown',
            expires_at: expiresAt || null
        };
        
        try {
            const result = await this.callExtendScript('writeLicenseData', [JSON.stringify(dataToStore)]);
            if (result && result.success) {
                 this.currentLicenseKey = licenseKey; 
                 this.currentLicenseType = licenseType;
                 this.currentLicenseExpiresAt = expiresAt;
                 console.log(`[License Store] Successfully wrote license (Type: ${licenseType}, Expires: ${expiresAt}) to file.`);
            } else {
                 console.error("[License Store] Failed to write license file:", result ? result.error : 'Unknown ExtendScript error');
            }
        } catch (error) {
            console.error("[License Store] Error calling ExtendScript to write license:", error);
        }
    }

    async clearStoredLicense() {
        try {
            const result = await this.callExtendScript('writeLicenseData', [JSON.stringify(null)]);
            if (result && result.success) {
                console.log("[License] Cleared stored license data from file.");
            } else {
                console.error("[License] Failed to clear license file:", result ? result.error : 'Unknown ExtendScript error');
            }
        } catch (error) {
             console.error("[License Store] Error calling ExtendScript to clear license:", error);
        }
        this.currentLicenseKey = null;
        this.currentLicenseType = null;
        this.currentLicenseExpiresAt = null;
    }

    async checkExistingLicense() {
        console.log("[License Check] Checking for existing license from file...");
        if (!this.machineId) {
            console.warn("[License Check] Machine ID not ready, generating again...");
            await this.generateMachineId();
        }
        console.log("[License Check] Current Machine ID:", this.machineId);
        
        let storedData = null;
        try {
            const result = await this.callExtendScript('readLicenseData');
            if (result && result.success) {
                storedData = result.data; 
                console.log("[License Check] Read from file:", storedData);
            } else {
                console.error("[License Check] Failed to read license file:", result ? result.error : 'ExtendScript read error');
                this._updateStatus(false, 'Error reading license data.', false, null, null);
                return;
            }
        } catch (error) {
             console.error("[License Check] Error calling ExtendScript to read license:", error);
             this._updateStatus(false, 'Error accessing license data.', false, null, null);
             return;
        }

        if (!storedData || !storedData.key) {
            console.log("[License Check] No valid license data found in file.");
            this._updateStatus(false, 'No license found.', false, null, null);
            return;
        }

        const { key: keyFromStorage, machineId: storedMachineId, validUntil, type: storedType, expires_at: storedExpiresAt } = storedData;
        
        try {
            console.log(`[License Check] Parsed from file - Key: ${keyFromStorage}, Type: ${storedType}, Expires: ${storedExpiresAt}, Stored Machine ID: ${storedMachineId}, Valid Until: ${validUntil ? new Date(validUntil).toISOString() : 'N/A'}`);
            
            if (storedMachineId !== this.machineId) {
                console.warn("[License Check] Machine ID mismatch in file. Stored license invalid.");
                await this.clearStoredLicense();
                this._updateStatus(false, 'Machine ID mismatch.', false, null, null);
                return;
            }

            const onlineValidationSuccess = await this.validateLicense(keyFromStorage);
            
            if (onlineValidationSuccess) {
                return; 
            }

            console.warn("[License Check] Online validation failed or network error. Checking offline cache...");
            let isExpired = false;
            if (storedExpiresAt) {
                try {
                   if (new Date(storedExpiresAt) < Date.now()) {
                       isExpired = true;
                       console.error("[License Check] Stored license has expired.");
                   }
                } catch { isExpired = true; }
            }

            if (!isExpired && validUntil && Date.now() < validUntil) {
                console.log("[License Check] Offline grace period is still valid.");
                this.currentLicenseKey = keyFromStorage;
                this.currentLicenseType = storedType;
                this.currentLicenseExpiresAt = storedExpiresAt;
                this._updateStatus(true, `Offline - valid until ${new Date(validUntil).toLocaleDateString()}`, true, storedType, storedExpiresAt);
            } else {
                console.error("[License Check] Offline grace period expired or invalid.");
                await this.clearStoredLicense();
                const message = isExpired ? 'License has expired.' : 'Offline period ended.';
                this._updateStatus(false, message, false, null, null);
            }

        } catch (error) {
            console.error('[License Check] Error processing license data:', error);
            await this.clearStoredLicense();
            this._updateStatus(false, 'Error processing license data.', false, null, null);
        }
    }

    async deactivateLicense() {
        if (!this.isLicensed || !this.currentLicenseKey || !this.machineId) {
            console.warn("[License Deactivate] No active license or machine ID to deactivate.");
            return;
        }
        
        console.log(`[License Deactivate] Deactivating key: ${this.currentLicenseKey} for machine: ${this.machineId}`);
        const keyBeingDeactivated = this.currentLicenseKey;

        try {
            const response = await fetch(this.deactivateApiEndpoint, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                },
                body: JSON.stringify({
                    licenseKey: this.currentLicenseKey,
                    machineId: this.machineId
                })
            });

            const data = await response.json();
            console.log("[License Deactivate] API Response Status:", response.status);
            console.log("[License Deactivate] API Response Data:", data);

            if (response.ok && data.success) {
                await this.clearStoredLicense();
                this._updateStatus(false, 'License Deactivated.', false, null, null);
                this.updateLicenseTabDisplay('inactive', `Key ${keyBeingDeactivated} deactivated.`, null, null);
            } else {
                 console.error("[License Deactivate] Deactivation failed on server.");
                 if (typeof this.onStatusChange === 'function') {
                    const currentStatus = this.isLicensed ? (this.isOfflineMode ? 'offline' : 'active') : 'inactive';
                    this.onStatusChange(currentStatus, `Deactivation Failed: ${data.message || 'Server Error'}`); 
                 }
            }

        } catch (error) {
            console.error("[License Deactivate] Network error...:", error);
             if (typeof this.onStatusChange === 'function') {
                const currentStatus = this.isLicensed ? (this.isOfflineMode ? 'offline' : 'active') : 'inactive';
                this.onStatusChange(currentStatus, 'Deactivation Failed: Network Error'); 
             }
        }
    }
}

// Timecode utilities
const TimecodeUtils = {
    dropFrameTimecodeToFrames(tc, framerate) {
        if (framerate !== 29.97 && framerate !== 59.94) {
            throw new Error('Drop frame timecode only supports 29.97 and 59.94 fps');
        }

        const fps = Math.round(framerate);
        const dropFrames = framerate === 29.97 ? 2 : 4;
        
        // Total minutes
        const totalMinutes = tc.hours * 60 + tc.minutes;
        
        // Calculate dropped frames
        const droppedFrames = dropFrames * (totalMinutes - Math.floor(totalMinutes / 10));
        
        // Regular frame count first
        const regularFrames = totalMinutes * 60 * fps + tc.seconds * fps + tc.frames;
        
        // Adjust for dropped frames
        return regularFrames - droppedFrames;
    },

    framesToDropFrameTimecode(frameCount, framerate) {
        if (framerate !== 29.97 && framerate !== 59.94) {
            throw new Error('Drop frame timecode only supports 29.97 and 59.94 fps');
        }

        const fps = Math.round(framerate);
        const dropFrames = framerate === 29.97 ? 2 : 4;
        
        // Calculate parts
        const framesPerMinute = fps * 60 - dropFrames; // for minutes not divisible by 10
        const framesPerTenMinutes = fps * 60 * 10 - dropFrames * 9; // dropped frames per 10 min
        
        // Ensure positive frame count
        frameCount = Math.max(0, frameCount);
        
        // Calculate parts
        const d = Math.floor(frameCount / framesPerTenMinutes);
        const m = frameCount % framesPerTenMinutes;
        
        let frames = frameCount;
        if (m > dropFrames) {
            frames = frameCount + (dropFrames * 9 * d) + dropFrames * Math.floor((m - dropFrames) / framesPerMinute);
        } else {
            frames = frameCount + dropFrames * 9 * d;
        }
        
        const f = frames % fps;
        const s = Math.floor((frames / fps) % 60);
        const min = Math.floor(frames / (fps * 60)) % 60;
        const h = Math.floor(frames / (fps * 3600));
        
        return { hours: h, minutes: min, seconds: s, frames: f };
    },

    timecodeToFrames(tc, framerate) {
        // For drop frame rates, use the special method
        if (framerate === 29.97 || framerate === 59.94) {
            return this.dropFrameTimecodeToFrames(tc, framerate);
        }
        
        // Non-drop frame calculation
        const fps = framerate;
        return Math.round(tc.hours * 3600 * fps + tc.minutes * 60 * fps + tc.seconds * fps + tc.frames);
    },

    framesToTimecode(frameCount, framerate) {
        // For drop frame rates, use the special method
        if (framerate === 29.97 || framerate === 59.94) {
            return this.framesToDropFrameTimecode(frameCount, framerate);
        }
        
        // Non-drop frame calculation
        const fps = framerate;
        const frames = frameCount % Math.round(fps);
        const seconds = Math.floor(frameCount / fps) % 60;
        const minutes = Math.floor(frameCount / (fps * 60)) % 60;
        const hours = Math.floor(frameCount / (fps * 3600));
        
        return { hours, minutes, seconds, frames };
    },

    formatTimecode(tc, isDropFrame = false) {
        const padHMS = num => num.toString().padStart(2, '0');

        const padF = num => {
            const rate = tc.frameRate || (isDropFrame ? 29.97 : 30);
            const maxFrames = Math.floor(rate) -1;
            const frameNum = num;
            const frames = Math.min(Math.max(0, frameNum), maxFrames);
            return frames.toString().padStart(2, '0');
        };
        
        const separator = isDropFrame ? ';' : ':';
        return `${padHMS(tc.hours)}:${padHMS(tc.minutes)}:${padHMS(tc.seconds)}${separator}${padF(tc.frames)}`;
    },

    parseFlexibleTimecode(timecodeString) {
        const tc = timecodeString.trim();
        let h = 0, m = 0, s = 0, f = 0;
        let match;

        match = tc.match(/^(\d{1,2})[:;](\d{1,2})[:;](\d{1,2})[:;](\d{1,2})$/);
        if (match) {
            [, h, m, s, f] = match.map(n => parseInt(n, 10));
            if ([h, m, s, f].some(isNaN)) throw new Error(`Invalid numbers in timecode: ${tc}`);
            return { hours: h, minutes: m, seconds: s, frames: f };
        }

        match = tc.match(/^(\d{1,2}):(\d{1,2})$/);
        if (match) {
            [, m, s] = match.map(n => parseInt(n, 10));
            f = 0;
            if ([m, s].some(isNaN)) throw new Error(`Invalid numbers in timecode: ${tc}`);
             return { hours: 0, minutes: m, seconds: s, frames: f };
        }

        if (tc.includes('.')) {
            const parts = tc.split('.');
            const p = parts.map(part => parseInt(part, 10));
            if (p.some(isNaN) || parts.some(part => !/^\d+$/.test(part)) || parts.length > 4 || parts.length < 1) {
                 throw new Error(`Invalid dot notation timecode: ${tc}`);
            }
            
             if (parts.length === 4) {
                 [h, m, s, f] = p;
             } else if (parts.length === 3) {
                 h = 0;
                 [m, s, f] = p;
             } else if (parts.length === 2) {
                 h = 0;
                 m = 0;
                 [s, f] = p;
             } else if (parts.length === 1) {
                 h = 0;
                 m = 0;
                 s = 0;
                 [f] = p;
             }
             return { hours: h, minutes: m, seconds: s, frames: f };
        }

        match = tc.match(/^(\d+)$/);
        if (match) {
            const digits = match[1];
            const len = digits.length;

            f = parseInt(digits.slice(-2), 10);
            s = len > 2 ? parseInt(digits.slice(-4, -2), 10) : 0;
            m = len > 4 ? parseInt(digits.slice(-6, -4), 10) : 0;
            h = len > 6 ? parseInt(digits.slice(-8, -6), 10) : 0;

            if (len <= 2) {
                 s = 0;
                 m = 0;
                 h = 0;
                 f = parseInt(digits, 10);
            }

            if ([h, m, s, f].some(isNaN)) {
                 throw new Error(`Invalid continuous digits timecode: ${tc}`);
            }
            return { hours: h, minutes: m, seconds: s, frames: f };
        }

        throw new Error(`Unrecognized timecode format: ${tc}`);
    }
};

class TimecodeCalculator {
    constructor() {
        this.frameRate = 29.97;
        this.targetFrameRate = 29.97;
        this.mode = 'duration';
        this.outputFormat = 'timecode';
        this.operation = 'add';
        this.roundFrames = true;
        this.removeZeros = true;
        this.manuallySetFrameRate = false;
        this._previousMode = this.mode;
        this.lastDurationResults = [];
        this.lastTrtResult = '';
        this.isUIEnabled = false;
        this.uiElementsToDisable = [];

        this.initializeUI();
        this.setupEventListeners();
        
        this.disableCalculatorUI("Checking license...");
    }

    initializeUI() {
        this.modeSelect = document.getElementById('mode-select');
        this.outputSelect = document.getElementById('output-select');
        this.operationSelect = document.getElementById('operation-select');
        this.timecodeInput = document.getElementById('timecode-input');
        this.resultDisplay = document.getElementById('result-display');
        this.calculateBtn = document.getElementById('calculate-btn');
        this.resetBtn = document.getElementById('reset-btn');
        this.copyResultsBtn = document.getElementById('copy-results');
        this.addCurrentTimecodeBtn = document.getElementById('add-current-timecode');
        this.addMarkersTimecodeBtn = document.getElementById('add-markers-timecode');
        this.copySegmentsBtn = document.getElementById('copy-segments');
        this.roundFramesCheckbox = document.getElementById('round-frames');
        this.removeZerosCheckbox = document.getElementById('remove-zeros');
        this.calculatorOptionsContainer = document.getElementById('calculator-options');
        this.durationOptionsContainer = document.getElementById('duration-options');
        this.durationCopyButtonsContainer = document.getElementById('duration-copy-buttons');
        this.copyDurationResultsBtn = document.getElementById('copy-duration-results');
        this.copyTrtBtn = document.getElementById('copy-trt');
        this.copyEmailBtn = document.getElementById('copy-email');
        this.targetFramerateContainer = document.getElementById('target-framerate-container');
        this.targetFramerateSelect = document.getElementById('target-framerate-select');

        if(this.modeSelect) this.modeSelect.value = this.mode;
        if(this.outputSelect) this.outputSelect.value = this.outputFormat;
        if(this.operationSelect) this.operationSelect.value = this.operation;
        if(this.roundFramesCheckbox) this.roundFramesCheckbox.checked = this.roundFrames;
        if(this.removeZerosCheckbox) this.removeZerosCheckbox.checked = this.removeZeros;
        if(this.targetFramerateSelect) this.targetFramerateSelect.value = this.targetFrameRate.toString();

        this.updatePlaceholder();

        this.getSequenceFrameRate();

        this.uiElementsToDisable = [
            this.modeSelect, this.outputSelect, this.operationSelect,
            this.timecodeInput, this.calculateBtn, this.resetBtn,
            this.copyResultsBtn, this.addCurrentTimecodeBtn,
            this.addMarkersTimecodeBtn, this.copySegmentsBtn,
            this.roundFramesCheckbox, this.removeZerosCheckbox,
            this.copyDurationResultsBtn, this.copyTrtBtn, this.copyEmailBtn,
            this.targetFramerateSelect
        ].filter(el => el);
    }

    setupEventListeners() {
        this.modeSelect.addEventListener('change', (e) => {
            this.mode = e.target.value;
            this.updateUI();
            this.updatePlaceholder();
            console.log('Mode changed to:', this.mode);
        });

        this.outputSelect.addEventListener('change', (e) => {
            this.outputFormat = e.target.value;
            this.updateUI();
            this.updatePlaceholder();
            console.log('Output format changed to:', this.outputFormat);
        });

        this.operationSelect.addEventListener('change', (e) => {
            this.operation = e.target.value;
            this.updatePlaceholder();
            console.log('Operation changed to:', this.operation);
        });

        if (this.targetFramerateSelect) {
            this.targetFramerateSelect.addEventListener('change', (e) => {
                this.targetFrameRate = parseFloat(e.target.value);
                console.log('Target frame rate changed to:', this.targetFrameRate);
            });
        }

        this.calculateBtn.addEventListener('click', async () => {
            this.calculate();
        });

        this.resetBtn.addEventListener('click', async () => {
            this.reset();
        });

        this.copySegmentsBtn.addEventListener('click', () => this.copySegments());

        this.copyResultsBtn.addEventListener('click', () => this.copyGeneralResults());

        this.copyDurationResultsBtn.addEventListener('click', () => this.copyDurationResults());
        this.copyTrtBtn.addEventListener('click', () => this.copyTrt());
        this.copyEmailBtn.addEventListener('click', () => this.copyEmail());

        this.addCurrentTimecodeBtn.addEventListener('click', async () => {
            await this.getSequenceFrameRate();
            this.addCurrentTimecode();
        });

        this.addMarkersTimecodeBtn.addEventListener('click', async () => {
            await this.getSequenceFrameRate();
            this.addAllMarkers();
        });

        this.roundFramesCheckbox.addEventListener('change', async (e) => {
            this.roundFrames = e.target.checked;
            console.log('Round frames:', this.roundFrames);
            if (this.mode === 'duration' && this.timecodeInput.value.trim()) {
                this.calculate(); 
            }
        });

        this.removeZerosCheckbox.addEventListener('change', async (e) => {
            this.removeZeros = e.target.checked;
            console.log('Remove zeros:', this.removeZeros);
            if (this.mode === 'duration' && this.timecodeInput.value.trim()) {
                this.calculate();
            }
        });
    }

    updateUI() {
        const isDurationMode = this.mode === 'duration';
        const isCalculatorMode = this.mode === 'calculator';
        const isConverterMode = this.mode === 'converter';
        const isTimecodeConversion = isConverterMode && this.outputFormat === 'timecode-conversion';

        // Update body class for CSS selectors
        if (isConverterMode) {
            document.body.classList.add('converter-mode');
        } else {
            document.body.classList.remove('converter-mode');
            // If not in converter mode but timecode-conversion is selected, reset to timecode
            if (this.outputFormat === 'timecode-conversion') {
                this.outputFormat = 'timecode';
                this.outputSelect.value = 'timecode';
            }
        }

        if (this.calculatorOptionsContainer) {
            this.calculatorOptionsContainer.style.display = isCalculatorMode ? 'flex' : 'none';
        }

        if (this.durationOptionsContainer) {
            this.durationOptionsContainer.style.display = isDurationMode ? 'flex' : 'none';
        }

        if (this.targetFramerateContainer) {
            this.targetFramerateContainer.style.display = isTimecodeConversion ? 'flex' : 'none';
        }

        if (this.copyResultsBtn) {
            this.copyResultsBtn.style.display = isDurationMode ? 'none' : 'inline-block';
        }
        if (this.durationCopyButtonsContainer) {
            this.durationCopyButtonsContainer.style.display = isDurationMode ? 'flex' : 'none';
        }

        if (this._previousMode && this._previousMode !== this.mode) {
            console.log(`Mode changed from ${this._previousMode} to ${this.mode}. Resetting input/output.`);
            this.reset();
        }
        this._previousMode = this.mode;
    }

    updatePlaceholder() {
        const examples = {
            duration: `Example: \n\n00;00;30;00\n00;10;53;25\n00;11;23;27\n00;20;19;03\n00;20;49;03\n00;31;23;02`,
            calculator: {
                add: `Example: \n\n00;49;28;09\n01;01;51;23`,
                subtract: `Example: \n\n01;01;51;23\n00;49;28;09`
            },
            converter: {
                timecode: `Example: \n\n01;01;51;23`,
                frames: `Example: \n\n111243`,
                'timecode-conversion': `Example: \n\n01;01;51;23`
            }
        };

        let placeholder = '';
        if (this.mode === 'duration') {
            placeholder = examples.duration;
        } else if (this.mode === 'calculator') {
            placeholder = examples.calculator[this.operation];
        } else if (this.mode === 'converter') {
            placeholder = examples.converter[this.outputFormat] || examples.converter.timecode;
        }

        this.timecodeInput.placeholder = placeholder;
    }

    async calculate() {
        try {
            this.checkLicense();
            
            await this.getSequenceFrameRate();
            
            const input = this.timecodeInput.value.trim();
            if (!input) {
                throw new Error('Please enter at least one timecode');
            }

            const timecodes = input.split('\n').filter(tc => tc.trim());
            if (timecodes.length === 0) {
                throw new Error('Please enter at least one valid timecode');
            }

            let result;
            if (this.mode === 'calculator') {
                result = this.calculateTimecodes(timecodes);
            } else if (this.mode === 'duration') {
                result = this.calculateDuration();
            } else {
                result = this.convertTimecodes(timecodes);
            }

            this.displayResults(timecodes, result);
        } catch (error) {
            console.error('Calculation error:', error);
            this.displayError(error.message);
        }
    }

    calculateDuration() {
        try {
            const timecodes = this.timecodeInput.value.trim().split('\n').filter(tc => tc.trim());
            if (timecodes.length < 2) {
                throw new Error('Please enter at least two timecodes');
            }

            let totalFrames = 0;
            const results = [];

            for (let i = 0; i < timecodes.length; i += 2) {
                const start = TimecodeUtils.parseFlexibleTimecode(timecodes[i]);
                const end = i + 1 < timecodes.length ? TimecodeUtils.parseFlexibleTimecode(timecodes[i + 1]) : null;
                
                if (!start || !end) {
                    throw new Error(`Invalid timecode format or missing end timecode for pair ${Math.floor(i/2) + 1}`);
                }

                console.log(`\nCalculating duration for pair ${Math.floor(i/2) + 1}:`);
                console.log('Start timecode:', start);
                console.log('End timecode:', end);

                const startFrames = this.isDropFrameRate(this.frameRate) ? 
                    TimecodeUtils.dropFrameTimecodeToFrames(start, this.frameRate) :
                    TimecodeUtils.timecodeToFrames(start, this.frameRate);
                
                const endFrames = this.isDropFrameRate(this.frameRate) ?
                    TimecodeUtils.dropFrameTimecodeToFrames(end, this.frameRate) :
                    TimecodeUtils.timecodeToFrames(end, this.frameRate);
                
                console.log('Start frames:', startFrames);
                console.log('End frames:', endFrames);

                const duration = endFrames - startFrames;
                console.log('Duration in frames:', duration);

                if (duration < 0) {
                    throw new Error(`Invalid duration: End time must be after start time in pair ${Math.floor(i/2) + 1}`);
                }

                totalFrames += duration;
                results.push(duration);
                console.log('Running total frames (raw):', totalFrames);
            }

            // If output format is frames, return the frame counts directly
            if (this.outputFormat === 'frames') {
                const formattedResults = [];
                for (let i = 0; i < results.length; i++) {
                    formattedResults.push(results[i].toString());
                }
                formattedResults.push(totalFrames.toString());
                return {
                    individualDurations: formattedResults.slice(0, -1),
                    totalTimecode: formattedResults[formattedResults.length - 1]
                };
            }

            // Continue with timecode formatting for non-frames output
            const formattedResults = [];
            let finalTotalFrames = 0;
            let processedRoundedTotal = false;

            for (let i = 0; i < results.length; i++) {
                const duration = results[i];
                console.log(`\nFormatting duration ${i + 1}:`, duration);

                let tcObj;
                const isDrop = this.isDropFrameRate(this.frameRate);
                if (isDrop) {
                    tcObj = TimecodeUtils.framesToDropFrameTimecode(duration, this.frameRate);
                } else {
                    tcObj = TimecodeUtils.framesToTimecode(duration, this.frameRate);
                }
                console.log('Duration as timecode object:', tcObj);

                let timecode = TimecodeUtils.formatTimecode({
                    ...tcObj,
                    frameRate: this.frameRate
                }, isDrop);
                console.log('Initially formatted duration:', timecode);

                let currentDurationFrames = duration;

                if (this.roundFrames) {
                    const [h, m, s, f] = timecode.split(/[:;]/).map(Number);
                    const halfFps = Math.floor(this.frameRate / 2);
                    let roundedTimecodeObj = { hours: h, minutes: m, seconds: s, frames: f };
                    
                    if (f >= halfFps) {
                        roundedTimecodeObj.seconds += 1;
                        roundedTimecodeObj.frames = 0;
                        if (roundedTimecodeObj.seconds >= 60) {
                            roundedTimecodeObj.minutes += Math.floor(roundedTimecodeObj.seconds / 60);
                            roundedTimecodeObj.seconds %= 60;
                        }
                        if (roundedTimecodeObj.minutes >= 60) {
                            roundedTimecodeObj.hours += Math.floor(roundedTimecodeObj.minutes / 60);
                            roundedTimecodeObj.minutes %= 60;
                        }
                        roundedTimecodeObj.hours %= 24;
                        
                    } else {
                        roundedTimecodeObj.frames = 0;
                    }
                    
                    timecode = TimecodeUtils.formatTimecode({
                        ...roundedTimecodeObj,
                        frameRate: this.frameRate
                    }, isDrop);

                    currentDurationFrames = isDrop ?
                        TimecodeUtils.dropFrameTimecodeToFrames(roundedTimecodeObj, this.frameRate) :
                        TimecodeUtils.timecodeToFrames(roundedTimecodeObj, this.frameRate);
                    console.log('Rounded duration frames:', currentDurationFrames);

                    if (!processedRoundedTotal) {
                        finalTotalFrames = currentDurationFrames;
                        processedRoundedTotal = true;
                    } else {
                        finalTotalFrames += currentDurationFrames;
                    }

                } else {
                     if (!processedRoundedTotal) {
                         finalTotalFrames += currentDurationFrames;
                     } else {
                         console.warn("Inconsistent state: accumulating raw frames after rounded frames?");
                     }
                }

                if (this.removeZeros) {
                    timecode = timecode.replace(/^00:/, '').replace(/^0:/, '');
                    if (this.roundFrames) {
                        timecode = timecode.replace(/[:;]00$/, '');
                    }
                }

                formattedResults.push(timecode);
                console.log('Final formatted duration for display:', timecode);
            }
            
            if (!processedRoundedTotal) {
                 finalTotalFrames = totalFrames;
            }

            console.log('\nFormatting total duration:');
            console.log('Final total frames for TRT (raw or rounded sum):', finalTotalFrames);

            let totalTimecode;
            const isTotalDrop = this.isDropFrameRate(this.frameRate);
            if (isTotalDrop) {
                const tcObj = TimecodeUtils.framesToDropFrameTimecode(finalTotalFrames, this.frameRate);
                totalTimecode = TimecodeUtils.formatTimecode({
                    ...tcObj,
                    frameRate: this.frameRate
                }, true);
            } else {
                const tcObj = TimecodeUtils.framesToTimecode(finalTotalFrames, this.frameRate);
                totalTimecode = TimecodeUtils.formatTimecode({
                    ...tcObj,
                    frameRate: this.frameRate
                }, false);
            }
            console.log('Final TRT timecode:', totalTimecode);

            formattedResults.push(totalTimecode);
            return {
                individualDurations: formattedResults.slice(0, -1),
                totalTimecode: formattedResults[formattedResults.length - 1]
            };
        } catch (error) {
            throw error;
        }
    }

    async getSequenceFrameRate() {
        try {
            if (!this.manuallySetFrameRate) {
                const frameRate = await new Promise((resolve) => {
                    csInterface.evalScript('getSequenceFrameRate()', (result) => {
                        resolve(parseFloat(result));
                    });
                });
                this.frameRate = frameRate;
            }
        } catch (e) {
            console.error('Error getting sequence frame rate:', e);
            this.frameRate = 29.97;
        }
    }

    isDropFrameRate(frameRate) {
        // Only 29.97 and 59.94 are drop frame rates
        return frameRate === 29.97 || frameRate === 59.94;
    }

    timecodeToFrames(timecode) {
        if (typeof timecode !== 'string' || timecode.trim() === '') {
            throw new Error('Invalid timecode input provided to timecodeToFrames');
        }
        const tc = TimecodeUtils.parseFlexibleTimecode(timecode);
        const isDropFrame = this.isDropFrameRate(this.frameRate);

        if (isDropFrame) {
            return TimecodeUtils.dropFrameTimecodeToFrames(tc, this.frameRate);
        } else {
            return TimecodeUtils.timecodeToFrames(tc, this.frameRate);
        }
    }

    convertTimecodes(timecodes) {
        return timecodes.map(tc => {
            if (this.outputFormat === 'frames') {
                // Convert the timecode to frames
                const frames = this.timecodeToFrames(tc);
                return frames.toString();
            } else if (this.outputFormat === 'timecode-conversion') {
                // For timecode-to-timecode conversion
                console.log(`Converting timecode ${tc} from ${this.frameRate}fps to ${this.targetFrameRate}fps`);
                
                try {
                    // Step 1: Parse the input timecode
                    const parsedTC = TimecodeUtils.parseFlexibleTimecode(tc);
                    console.log('Parsed timecode:', parsedTC);
                    
                    // Step 2: Convert to frames using source frame rate
                    const frames = TimecodeUtils.timecodeToFrames(parsedTC, this.frameRate);
                    console.log('Frames at source rate:', frames);
                    
                    // Step 3: Convert frames back to timecode using target frame rate
                    const targetTC = TimecodeUtils.framesToTimecode(frames, this.targetFrameRate);
                    console.log('Target timecode object:', targetTC);
                    
                    // Step 4: Format the timecode
                    const isTargetDropFrame = this.isDropFrameRate(this.targetFrameRate);
                    const formattedTC = TimecodeUtils.formatTimecode({
                        ...targetTC,
                        frameRate: this.targetFrameRate
                    }, isTargetDropFrame);
                    
                    console.log('Final formatted timecode:', formattedTC);
                    return formattedTC;
                } catch (error) {
                    console.error('Error in timecode conversion:', error);
                    return `Error: ${error.message}`;
                }
            } else {
                // Regular timecode conversion (same frame rate)
                const frames = this.timecodeToFrames(tc);
                return this._framesToTimecodeInternal(frames);
            }
        });
    }

    calculateTimecodes(timecodes) {
        if (timecodes.length < 2) {
            throw new Error('Calculator mode requires at least 2 timecodes');
        }

        const frames = timecodes.map(tc => this.timecodeToFrames(tc));
        console.log('Timecodes converted to frames:', frames);

        let result;
        if (this.operation === 'add') {
            result = frames.reduce((sum, current) => sum + current, 0);
            console.log('Sum of frames:', result);
        } else {
            result = frames.reduce((diff, current, index) => {
                return index === 0 ? current : diff - current;
            });
            console.log('Difference of frames:', result);
        }

        return this.outputFormat === 'frames' ? 
            result.toString() : 
            this._framesToTimecodeInternal(result);
    }

    _framesToTimecodeInternal(totalFrames) {
        try {
            console.log('Converting frames back to timecode:', totalFrames, 'at frame rate:', this.frameRate);
            const isDropFrame = this.isDropFrameRate(this.frameRate);
            let tcObj;
            if (isDropFrame) {
                console.log('Using drop-frame conversion');
                tcObj = TimecodeUtils.framesToDropFrameTimecode(totalFrames, this.frameRate);
            } else {
                console.log('Using non-drop-frame conversion');
                tcObj = TimecodeUtils.framesToTimecode(totalFrames, this.frameRate);
            }
            console.log('Converted to timecode object:', tcObj);
            const result = TimecodeUtils.formatTimecode({...tcObj, frameRate: this.frameRate}, isDropFrame);
            console.log('Final formatted timecode:', result);
            return result;
        } catch (error) {
            throw new Error(`Error converting frames to timecode: ${error.message}`);
        }
    }

    displayResults(inputs, results) {
        console.log('Displaying results. Mode:', this.mode);
        console.log('Inputs:', inputs);
        console.log('Results:', results);
        
        if (typeof inputs === 'string') {
            this.resultDisplay.innerHTML = `<pre style="white-space: pre-line;">${inputs}</pre>`;
            return;
        }
        
        let output;
        if (this.mode === 'calculator') {
            output = results;
        } else if (this.mode === 'duration') {
            this.lastDurationResults = results.individualDurations || [];
            this.lastTrtResult = results.totalTimecode || '';
            
            // Just show individual durations without the total when in frames mode
            output = this.lastDurationResults.join('\n');
        } else {
            output = results.join('\n');
            
            // Add frame rate information when using timecode-to-timecode conversion
            if (this.mode === 'converter' && this.outputFormat === 'timecode-conversion') {
                const sourceFrameRate = this.frameRate.toFixed(this.frameRate % 1 === 0 ? 0 : 2);
                const targetFrameRate = this.targetFrameRate.toFixed(this.targetFrameRate % 1 === 0 ? 0 : 2);
                output = `Converted from ${sourceFrameRate}fps to ${targetFrameRate}fps:\n\n${output}`;
            }
        }
        
        console.log('Final output:', output);
        this.resultDisplay.innerHTML = `<pre style="white-space: pre-line;">${output}</pre>`;
    }

    displayError(message) {
        this.resultDisplay.innerHTML = `<div style="color: var(--uxp-host-alert-color)">${message}</div>`;
    }

    reset() {
        this.timecodeInput.value = '';
        this.resultDisplay.innerHTML = '';
        this.lastDurationResults = [];
        this.lastTrtResult = '';
    }

    async addCurrentTimecode() {
        try {
            this.checkLicense();
            
            const result = await new Promise((resolve) => {
                csInterface.evalScript('getCurrentTimecode()', (rawResult) => {
                    console.log("Raw result from getCurrentTimecode:", rawResult);
                    try {
                        const parsedResult = JSON.parse(rawResult);
                        console.log("Parsed result from getCurrentTimecode:", parsedResult);
                        resolve(parsedResult);
                    } catch (parseError) {
                        console.error("Failed to parse JSON from getCurrentTimecode:", parseError);
                        console.error("Raw string was:", rawResult);
                        resolve(null);
                    }
                });
            });

            console.log("Result object after Promise:", result);

            if (result && result.timecode) {
                console.log("Timecode found:", result.timecode);
                if (this.timecodeInput.value.trim()) {
                    this.timecodeInput.value += '\n';
                }
                this.timecodeInput.value += result.timecode;
            } else {
                console.log("Result object did not contain expected timecode property.");
                this.displayError('Failed to get timecode or unexpected format received. Please check console.');
            }
        } catch (error) {
            console.error('Error in addCurrentTimecode:', error);
            this.displayError('Failed to get current timecode. Please make sure a sequence is open in Premiere Pro.');
        }
    }

    async copyGeneralResults() {
        this._copyToClipboard(this.resultDisplay.textContent, 'Results copied to clipboard!');
    }

    copyDurationResults() {
        try {
            this.checkLicense();
            if (!this.lastDurationResults || this.lastDurationResults.length === 0) {
                this.displayError('No duration results to copy.');
                return;
            }
            const textToCopy = this.lastDurationResults.join('\n');
            this._copyToClipboard(textToCopy, 'Individual durations copied!');
        } catch (error) {
            this.displayError(error.message);
        }
    }

    copyTrt() {
        try {
            this.checkLicense();
            if (!this.lastTrtResult) {
                this.displayError('No TRT result to copy.');
                return;
            }
            this._copyToClipboard(this.lastTrtResult, 'TRT copied!');
        } catch (error) {
            this.displayError(error.message);
        }
    }

    copyEmail() {
        try {
            this.checkLicense();
            if (!this.lastTrtResult || !this.lastDurationResults || this.lastDurationResults.length === 0) {
                this.displayError('No duration results to format for email.');
                return;
            }
            const numberedDurations = this.lastDurationResults
                .map((duration, index) => `${index + 1}. ${duration}`)
                .join('\n');
            const textToCopy = `TRT: ${this.lastTrtResult}\n\n${numberedDurations}`;
            this._copyToClipboard(textToCopy, 'Email format copied!');
        } catch (error) {
            this.displayError(error.message);
        }
    }

    _copyToClipboard(text, successMessage) {
        try {
             if (!text || !text.trim()) {
                 throw new Error('Nothing to copy.');
             }

            const textArea = document.createElement('textarea');
            textArea.value = text;

            textArea.style.position = 'absolute';
            textArea.style.left = '-9999px';
            textArea.style.top = '0';
            document.body.appendChild(textArea);

            textArea.select();
            textArea.setSelectionRange(0, 99999);
            const successful = document.execCommand('copy');

            document.body.removeChild(textArea);

            if (!successful) {
                throw new Error('Failed to copy text to clipboard.');
            }
            
            const originalContent = this.resultDisplay.innerHTML;
            this.resultDisplay.innerHTML = `<div style="color: var(--uxp-host-positive-color); padding: 5px 0;">${successMessage}</div>`; 
            setTimeout(() => {
                if (this.resultDisplay.innerHTML.includes(successMessage)) { 
                   this.resultDisplay.innerHTML = originalContent; 
                }
            }, 1500);
        } catch (error) {
            console.error('Copy error:', error);
            this.displayError(error.message);
        }
    }

    async addAllMarkers() {
        try {
            this.checkLicense();
            
            console.log('Starting addAllMarkers...');
            const result = await new Promise((resolve) => {
                csInterface.evalScript('getMarkersTimecode()', (result) => {
                    console.log('Raw result from ExtendScript:', result);
                    
                    try {
                        const parsed = JSON.parse(result);
                        console.log('Parsed result:', parsed);
                        resolve(parsed);
                    } catch (parseError) {
                        console.error('Error parsing JSON:', parseError);
                        console.error('Invalid JSON string:', result);
                        resolve(null);
                    }
                });
            });

            console.log('Result object type:', typeof result);
            console.log('Result object:', result);
            console.log('Result.success:', result?.success);
            console.log('Result.markers:', result?.markers);
            console.log('Result.markers length:', result?.markers?.length);

            if (result && result.success && result.markers && result.markers.length > 0) {
                const validMarkers = result.markers.filter(marker => {
                    console.log('Checking marker:', marker);
                    console.log('Marker timecode:', marker?.timecode);
                    console.log('Timecode components:', {
                        hours: typeof marker?.timecode?.hours,
                        minutes: typeof marker?.timecode?.minutes,
                        seconds: typeof marker?.timecode?.seconds,
                        frames: typeof marker?.timecode?.frames
                    });
                    
                    return marker && 
                           marker.timecode && 
                           typeof marker.timecode.hours === 'number' &&
                           typeof marker.timecode.minutes === 'number' &&
                           typeof marker.timecode.seconds === 'number' &&
                           typeof marker.timecode.frames === 'number';
                });

                console.log('Number of valid markers found:', validMarkers.length);
                console.log('Valid markers:', validMarkers);

                if (validMarkers.length === 0) {
                    this.displayError('No valid markers found in the active sequence. Please add some markers to your sequence first.');
                    return;
                }

                const timecodes = validMarkers.map(marker => {
                    const separator = marker.timecode.isDropFrame ? ';' : ':';
                    const pad = num => num.toString().padStart(2, '0');
                    return `${pad(marker.timecode.hours)}:${pad(marker.timecode.minutes)}:${pad(marker.timecode.seconds)}${separator}${pad(marker.timecode.frames)}`;
                }).join('\n');
                
                if (this.timecodeInput.value.trim()) {
                    this.timecodeInput.value += '\n';
                }
                this.timecodeInput.value += timecodes;
            } else {
                this.displayError('No markers found in the active sequence. Please add some markers to your sequence first.');
            }
        } catch (error) {
            console.error('Error getting markers:', error);
            this.displayError('Failed to get markers. Please make sure you have a sequence open in Premiere Pro.');
        }
    }

    copySegments() {
        try {
            this.checkLicense();
            
            const input = this.timecodeInput.value.trim();
            if (!input) {
                throw new Error('No timecodes to copy.');
            }

            const timecodes = input.split('\n').filter(tc => tc.trim());
            if (timecodes.length === 0) {
                throw new Error('No valid timecodes to copy.');
            }

            const segments = [];
            for (let i = 0; i < timecodes.length; i += 2) {
                const segment = [];
                if (timecodes[i]) {
                    segment.push(timecodes[i]);
                }
                if (timecodes[i + 1]) {
                    segment.push(timecodes[i + 1]);
                }
                if (segment.length > 0) {
                    segments.push(segment.join('\t')); // Use tab as separator for Excel
                }
            }

            const formattedText = segments.join('\n');
            this._copyToClipboard(formattedText, 'Segments copied to clipboard!');
        } catch (error) {
            console.error('Copy segments error:', error);
            this.displayError(error.message);
        }
    }

    handleLicenseStatusChange(status, message) {
        const isValid = (status === 'active' || status === 'offline');
        console.log(`[Calculator] Received license status: ${status}, Valid: ${isValid}`);
        
        if (isValid) {
            this.enableCalculatorUI();
            this.updateUI(); 
        } else {
            this.disableCalculatorUI(message || "License inactive or invalid.");
            if (status === 'inactive' || status === 'error') {
                 window.licenseManager?.showActivationSection();
            }
        }
    }
    
    enableCalculatorUI() {
        console.log("[Calculator] Enabling UI elements.");
        this.uiElementsToDisable.forEach(el => {
            el.disabled = false;
            el.style.opacity = 1;
            el.style.cursor = 'pointer'; 
        });
         document.querySelectorAll('.spectrum-checkbox-label').forEach(label => label.style.opacity = 1);
        this.isUIEnabled = true;
    }
    
    disableCalculatorUI(reason) {
        console.log(`[Calculator] Disabling UI elements. Reason: ${reason}`);
        this.uiElementsToDisable.forEach(el => {
            el.disabled = true;
            el.style.opacity = 0.5;
            el.style.cursor = 'not-allowed';
        });
         document.querySelectorAll('.spectrum-checkbox-label').forEach(label => label.style.opacity = 0.5);
        this.isUIEnabled = false;
    }

    checkLicense() {
        if (!this.isUIEnabled) {
            throw new Error('Functionality disabled. Please activate your license.');
        }
    }
}

async function initializeApp() { 
    console.log("[Init] Starting App Initialization...");
    try {
        const licenseManager = new LicenseManager();
        const calculator = new TimecodeCalculator();
        
        window.licenseManager = licenseManager;
        window.calculator = calculator;

        licenseManager.onStatusChange = (status, message) => {
            console.log(`[Init Callback] License status changed: ${status}, Message: ${message}`);
            calculator.handleLicenseStatusChange(status, message);
            licenseManager.updateLicenseTabDisplay(status, message, licenseManager.currentLicenseType, licenseManager.currentLicenseExpiresAt);
        };

        licenseManager.initialize().then(() => {
            console.log("[Init] Background license check process completed.");
        }).catch(error => {
             console.error("[Init] Error during background license check:", error);
             calculator.handleLicenseStatusChange('error', 'License check failed');
             licenseManager.updateLicenseTabDisplay('error', 'License check failed', null, null);
             licenseManager.showActivationSection();
        });

        console.log("[Init] UI Initialized, background check running.");

    } catch (error) {
        console.error('Failed to initialize app:', error);
    }
}

if (document.readyState === 'loading') {
    document.addEventListener('DOMContentLoaded', () => {
        initializeApp().catch(error => {
            console.error('Error during initialization:', error);
        });
    });
} else {
    initializeApp().catch(error => {
        console.error('Error during initialization:', error);
    });
}
